Attribute VB_Name = "modADSmartIO"
'---------------------------------
'--- ADSmartIO.bas
'---
'--- Applied Data Systems
'--- Revision History:
'---   February 27, 2002
'---     C. Tacke
'---     Version 1.0 completed
'---
'---------------------------------

Option Explicit

' ----------------------- WINDOWS API DECLARATIONS -----------------------
Public Declare Function GetLastError Lib "coredll.dll" () As Long
Public Declare Function CreateFile Lib "coredll.dll" Alias "CreateFileW" ( _
                        ByVal lpFileName As String, _
                        ByVal dwDesiredAccess As Long, _
                        ByVal dwShareMode As Long, _
                        ByVal lpSecurityAttributes As Long, _
                        ByVal dwDisposition As Long, _
                        ByVal dwFlags As Long, _
                        ByVal hTemplate As Long) As Long
Public Declare Function DeviceIoControl Lib "coredll.dll" ( _
                        ByVal hDevice As Long, _
                        ByVal dwIoControlCode As Long, _
                        ByVal lpInBuffer As String, _
                        ByVal nInBufferSize As Long, _
                        ByVal lpOutBuffer As String, _
                        ByVal nOutBufferSize As Long, _
                        lpBytesReturned As Long, _
                        lpOverlapped As Long) As Long
Public Declare Function CloseHandle Lib "coredll.dll" (hObject As Long) As Long

' ---------------------- ADSMARTIO API DECLARATIONS --------------------------
' -------                  ****** NOTE !!! ******                       ------
' ------- The following functions are declared as being in smartio.dll. ------
' ------- For the Graphics Client Plus, they are in keybddr.dll so if   ------
' ------- you get an error, replace all occurrences of smartio.dll with ------
' ------- keybddr.dll within this declarations section.                 ------
'-----------------------------------------------------------------------------
Declare Function SIOFirmwareVersion Lib "smartio.dll" () As Long
Declare Function SIOReadDeviceVersion Lib "smartio.dll" () As Long
Declare Function SIOReadDevicePartType Lib "smartio.dll" () As Long

Declare Function SIOReadKeypad Lib "smartio.dll" () As Byte
Declare Function SIOSetKeypadSize Lib "smartio.dll" (ByVal x As Byte, ByVal y As Byte) As Integer
Declare Function SIOSetSpeaker Lib "smartio.dll" (ByVal Duration As Byte, ByVal tone As Byte) As Integer

Declare Function SIOAnalogConversion Lib "smartio.dll" (ByVal Channel As Byte) As Long
Declare Function SIOReadVoltage Lib "smartio.dll" () As Integer
Declare Function SIOReadTemperature Lib "smartio.dll" () As Integer

Declare Sub SIOSelectOption Lib "smartio.dll" (ByVal bOption As Byte)

' --------  Following functions not supported by Graphics Master --------------
' --------  Use DeviceIOControl instead                          --------------
Declare Sub SIOBackLightPwm Lib "smartio.dll" (ByVal Value As Byte)
' Note: documentaion incorrectly identifies this as SIOVEEPwm
Declare Sub SIOBackLightVEEPwm Lib "smartio.dll" (ByVal Value As Byte)
Declare Sub SIOBackLightON Lib "smartio.dll" ()
Declare Sub SIOBackLightOFF Lib "smartio.dll" ()
'------------------------------------------------------------------------------

Declare Sub SIOWritePortA Lib "smartio.dll" (ByVal Value As Byte)
Declare Sub SIOWritePortB Lib "smartio.dll" (ByVal Value As Byte)
Declare Sub SIOWritePortC Lib "smartio.dll" (ByVal Value As Byte)
Declare Sub SIOWritePortD Lib "smartio.dll" (ByVal Value As Byte)

Declare Sub SIOInitDDRA Lib "smartio.dll" (ByVal Value As Byte)
Declare Sub SIOInitDDRB Lib "smartio.dll" (ByVal Value As Byte)
Declare Sub SIOInitDDRC Lib "smartio.dll" (ByVal Value As Byte)
Declare Sub SIOInitDDRD Lib "smartio.dll" (ByVal Value As Byte)

Declare Function SIOReadPortA Lib "smartio.dll" () As Long
Declare Function SIOReadPortB Lib "smartio.dll" () As Long
Declare Function SIOReadPortC Lib "smartio.dll" () As Long
Declare Function SIOReadPortD Lib "smartio.dll" () As Long

'----- The following functions are defined in the ADSmartIO header ------
'----- but are not implemented here                                ------
Declare Function SIOSetSSPTimeout Lib "smartio.dll" (ByVal timeout As Long) As Integer
Declare Function SIOArmVibSwitch Lib "smartio.dll" () As Integer
Declare Function SIOVibrateDetection Lib "smartio.dll" () As Integer
Declare Function SIORadioDetection Lib "smartio.dll" () As Integer
Declare Function SIOSetVibDetTimeout Lib "smartio.dll" (ByVal timeout As Long) As Integer
Declare Function SIOSetRadioDetTimeout Lib "smartio.dll" (ByVal timeout As Long) As Integer

Declare Function SIOWriteEEPROM Lib "smartio.dll" (ByVal blockNumber As Byte, ByVal eepromData As String) As Integer
Declare Function SIOReadEEPROM Lib "smartio.dll" (ByVal blockNumber As Byte, ByVal eepromData As String) As Integer

'-------------- CONSTANT DEFINITIONS ---------------
Public Const GENERIC_READ As Long = &H80000000
Public Const GENERIC_WRITE As Long = &H40000000
Public Const OPEN_EXISTING As Long = 3
Public Const IOCTL_SET_BACKLIGHT_ON As Long = &H2
Public Const IOCTL_SET_BACKLIGHT_OFF As Long = &H3
Public Const IOCTL_SET_BACKLIGHT_LEVEL As Long = &H4
Public Const IOCTL_SET_VEE_LEVEL As Long = &H14

Private bSAM1Open As Boolean
Private hSAM1 As Long

Private Function ToBinaryString(Number As Variant, Bytes As Integer) As String
    Dim i As Integer
    Dim bIsNegative As Boolean

    ' cannot check byte positions other than 0 to 3
    If Bytes > 4 Or Bytes < 1 Then Exit Function

    ' If the number is negative, we need to handle it last, so we'll set a flag
    If Number < 0 Then
        bIsNegative = True
        
        ' get the absolute value
        Number = Number * -1
        
        ' get the binary complement
        Number = Number Xor ((2 ^ (8 * Bytes - 1)) - 1)
        
        ' add one
        Number = Number + 1
    End If

    ' start at the least significant bit (0) and work backwards
    For i = 0 To Bytes - 1
        If i = Bytes - 1 And bIsNegative Then
            ' If the number is negative we must turn on the most significant bit
            ' and then and append it to the string
            ToBinaryString = ToBinaryString & (ChrB(GetByteValue(Number, i) + &H80))
        Else
            ' just append the byte to our string
            ToBinaryString = ToBinaryString & ChrB(GetByteValue(Number, i))
        End If
    Next i
End Function

Private Function GetByteValue(Number As Variant, BytePos As Integer) As Integer
    Dim mask As Long
    
    On Error Resume Next
    
    ' cannot check byte positions other than 0 to 3
    If BytePos > 3 Or BytePos < 0 Then Exit Function
        
    If BytePos < 3 Then
        ' build a mask of all bits on for the desired byte
        mask = &HFF * (2 ^ (8 * BytePos))
    Else
        ' the last bit is reserved for sign (+/-)
        mask = &H7F * (2 ^ (8 * BytePos))
    End If
    
    ' turn of all bits but the byte we're after
    GetByteValue = Number And mask
    
    ' move that byte to the end of the number
    GetByteValue = GetByteValue / (2 ^ (8 * BytePos))
End Function

Public Function ADSFirmwareVersion() As Long
    ' returns 16-bit firmware version from ATMEL processor
    
    ADSFirmwareVersion = SIOFirmwareVersion()
End Function

Public Function ADSDeviceVersion() As String
    ' returns the ASCII character of the SMART IO micro code revision level
    
    ADSDeviceVersion = Chr(SIOReadDeviceVersion())
End Function

Public Function ADSVoltage() As Long
    ' returns numeric representation of voltage from 0 to 123
    
    ADSVoltage = SIOReadVoltage()
End Function

Public Function ADSTemperature() As Long
    ' returns numeric representation of temperature
     
    ADSTemperature = SIOReadTemperature()
End Function

Public Function ADSDevicePartType() As Long
    ' returns ATMEL micro-controller part type
    
    ADSDevicePartType = SIOReadDevicePartType()
End Function

Public Function ADSReadKeypad() As Byte
    ' return values depend on firmware version
    ' see ADSmartIO Driver documentation
    
    ADSReadKeypad = SIOReadKeypad()
End Function

Public Function ADSSetKeypadSize(Width As Byte, Height As Byte) As Boolean
    ' set the size of the keypad
    ' see ADSmartIO Driver documentation for complete explanation
    
    Dim ret As Integer
    
    ret = SIOSetKeypadSize(Width, Height)
    
    If ret = 0 Then
        ADSSetKeypadSize = False
    Else
        ADSSetKeypadSize = True
    End If
End Function

Public Sub ADSBacklight(Lit As Boolean)
    ' Not supported on Graphics Master!
    On Error Resume Next
    
    If Lit Then
        SIOBackLightON
    Else
        SIOBackLightOFF
    End If
    
    If Err.Number Then MsgBox GetLastError(), vbCritical, "Error"
End Sub

Public Sub ADSGMasterBacklight(Lit As Boolean)
    On Error Resume Next
    
    Dim hDevice As Long
    Dim inbuffer As String
    Dim outbuffer As String
    Dim bytesreturned As Long
    Dim ret As Long
    
    ' initialize buffers
    inbuffer = String(2, 0)
    outbuffer = String(2, 0)
       
    ' open device
    hDevice = CreateFile("SAM1:", GENERIC_WRITE, 0, 0, OPEN_EXISTING, 0, 0)
    
    ' check for success
    If hDevice = 0 Then
        MsgBox "Failed to open device!", vbExclamation, "Error"
        Exit Sub
    End If
    
    ' turn on/off backlight
    If Lit Then
        ret = DeviceIoControl(hDevice, IOCTL_SET_BACKLIGHT_ON, inbuffer, LenB(inbuffer), outbuffer, LenB(outbuffer), bytesreturned, 0)
    Else
        ret = DeviceIoControl(hDevice, IOCTL_SET_BACKLIGHT_OFF, inbuffer, LenB(inbuffer), outbuffer, LenB(outbuffer), bytesreturned, 0)
    End If
    
    ' check for success
    If ret = 0 Then
        MsgBox "Device IO call failed!", vbExclamation, "Error"
    End If
    
    ' close device
    CloseHandle hDevice
    
    If Err.Number Then MsgBox GetLastError(), vbCritical, "Error"
End Sub

Public Function ADSSetBackLightLevel(Level As Integer) As Boolean
    ' Not supported on Graphics Master!
    If Level > 255 Or Level < 0 Then
        ADSSetBackLightLevel = False
        Exit Function
    End If
    
    ADSSetBackLightLevel = True
    
    SIOBackLightPwm CByte(Level)
End Function

Public Function ADSSetContrast(Level As Integer) As Boolean
    ' Not supported on Graphics Master!
    If Level > 255 Or Level < 0 Then
        ADSSetContrast = False
        Exit Function
    End If
    
    ADSSetContrast = True
    
    SIOBackLightVEEPwm CByte(Level)
End Function

Public Function ADSSetGMasterBackLightLevel(NewLevel As Integer) As Boolean
    ADSSetGMasterBackLightLevel = ADSSetSAM1(NewLevel, IOCTL_SET_BACKLIGHT_LEVEL)
End Function

Public Function ADSSetGMasterContrast(NewLevel As Integer) As Boolean
    ADSSetGMasterBackLightLevel = ADSSetSAM1(NewLevel, IOCTL_SET_VEE_LEVEL)
End Function

Private Function ADSSetSAM1(NewLevel As Integer, ControlCode As Integer) As Boolean
    On Error Resume Next
    
    Dim inbuffer As String
    Dim outbuffer As String
    Dim bytesreturned As Long
    Dim ret As Long
    
    If NewLevel > 255 Or NewLevel < 0 Then
        ADSSetSAM1 = False
        Exit Function
    End If
        
    ADSSetSAM1 = True
    
    hSAM1 = 0
    
    ' initialize buffers
    inbuffer = ToBinaryString(NewLevel, 2)
    outbuffer = String(2, 0)
    
    ' open device
    If Not bSAM1Open Then
        hSAM1 = CreateFile("SAM1:", GENERIC_WRITE, 0, 0, OPEN_EXISTING, 0, 0)
        bSAM1Open = True
    End If
    
    ' check for success
    If hSAM1 = 0 Then
        MsgBox "Failed to open device!", vbExclamation, "Error " & GetLastError
        ADSSetSAM1 = False
        bSAM1Open = False
        Exit Function
    End If
    
    ret = DeviceIoControl(hSAM1, ControlCode, inbuffer, LenB(inbuffer), outbuffer, LenB(outbuffer), bytesreturned, 0)
    
    ' check for success
    If ret = 0 Then
        ADSSetSAM1 = False
        MsgBox "Device IO call failed!", vbExclamation, "Error" & GetLastError
    End If
    
    ' close device
    ret = CloseHandle(hSAM1)
    
    bSAM1Open = False
    If Err.Number Then MsgBox Err.Description, vbCritical, "Error"
End Function

Public Function ADSPlayTone(Duration As Integer, Frequency As Long) As Boolean
    ' duration in milliseconds
    ' frequency in hertz
    Dim dur As Integer
    Dim tone As Integer
    
    ADSPlayTone = True
    
    ' do conversions
    dur = Round(Duration / 50 - 1, 0)
    
    If dur < 0 Or dur > 255 Then
        ADSPlayTone = False
        Exit Function
    End If
    
    tone = Round((1 / (Frequency * 34.72)) * 1000000 - 1, 0)

    If tone < 0 Or tone > 255 Then
        ADSPlayTone = False
        Exit Function
    End If
    
    ADSPlayTone = SIOSetSpeaker(CByte(dur), CByte(tone))
    If ADSPlayTone Then
        MsgBox "Played " & tone & " for " & dur
    End If
End Function

Public Function ADSWritePort(Port As Integer, Value As Integer) As Boolean
    If Value < 0 Or Value > 255 Then
        ADSWritePort = False
        Exit Function
    End If
    
    ADSWritePort = True
    
    Select Case Port
        Case 0
            SIOWritePortA CByte(Value)
        Case 1
            SIOWritePortB CByte(Value)
        Case 2
            SIOWritePortC CByte(Value)
        Case 3
            SIOWritePortD CByte(Value)
        Case Else
            ADSWritePort = False
    End Select
End Function

Public Function ADSReadPort(Port As Integer) As Integer
    Select Case Port
        Case 0
            ADSReadPort = SIOReadPortA()
        Case 1
            ADSReadPort = SIOReadPortB()
        Case 2
            ADSReadPort = SIOReadPortC()
        Case 3
            ADSReadPort = SIOReadPortD()
        Case Else
            ADSReadPort = -1
    End Select
End Function

